@@ -7,13 +7,15 @@ use warnings;
 use HTML::Entities; # For encode_entities()
 use Apache2::Cookie;
 use Data::Dumper;
+use Time::HiRes qw(gettimeofday);
+use HTTP::Date;
 
 use Masterkey::Admin::Session;
 use Masterkey::Admin::Config;
 use Masterkey::Admin::Torus;
 
 our @ISA = qw();
-our $VERSION = '3.30';
+our $VERSION = '4.6';
 
 sub new {
     my $class = shift();
@@ -94,7 +96,7 @@ sub fieldsWithDefaultValues {
 
 sub new_with_session {
     my $class = shift();
-    my($tabName, $r, $nouser) = @_;
+    my($tabName, $r, $nouser, $identityId, $force_uid) = @_;
 
     my $admin;
     eval {
@@ -106,7 +108,11 @@ sub new_with_session {
     my $domain = $admin->conf("authDomain");
     my $session = undef;
     my $cookies = Apache2::Cookie->fetch($r);
-    my $cookie = $cookies->{"session-$domain"};
+    my $cname = "session-$domain";
+    if ($ENV{MAnewurls}) {
+	$cname .= "-" . $identityId;
+    }
+    my $cookie = $cookies->{$cname};
     if (defined $cookie) {
 	my $cval = $cookie->value();
 	$session = Masterkey::Admin::Session->find($cval);
@@ -116,30 +122,31 @@ sub new_with_session {
     if (!defined $session) {
 	$session = Masterkey::Admin::Session->new();
 	my $cookie = new Apache2::Cookie($r, -path => "/",
-					 -name => "session-$domain",
+					 -name => $cname,
 					 -value => $session->cookie());
 	$cookie->bake($r);
     }
 
+    if (defined $force_uid) {
+	warn "forcing new session's ID to $force_uid";
+	$session->update(user_id => $force_uid);
+    }
+
     my $user = undef;
     goto DONE if $nouser;
 
     my $uid = $session->user_id();
 
-    # On http://ohiolink-test.indexdata.com/ this fails randomly
-    # about half the time, saying "Undefined subroutine
-    # &Apache2::Request called at
-    # /usr/lib/perl5/site_perl/5.8.8/Masterkey/Admin.pm line 115".
-    # We don't understand why, so for now we just comment it out.
-#    if (!$uid) {
-#	# Attempt an immediate login with supplied parameters, if any
-#	my $usefulR = new Apache2::Request($r);
-#	my $user = $usefulR->param("authUser");
-#	my $password = $usefulR->param("authPassword");
-#	if (defined $user && defined $password) {
-#	    warn "### we should log in as $user/$password";
-#	}
-#    }
+    if (!$uid) {
+	use Apache2::Request;
+	# Attempt an immediate login with supplied parameters, if any
+	my $usefulR = Apache2::Request->new($r);
+	my $user = $usefulR->param("authUser");
+	my $password = $usefulR->param("authPassword");
+	if (defined $user && defined $password) {
+	    warn "### we should log in as $user/$password";
+	}
+    }
 
     if (!$uid) {
 	# Check whether we were logged into the Harvester Console,
@@ -159,28 +166,78 @@ sub new_with_session {
 	}
     }
 
+    my %criteria;
     if ($uid) {
-	eval {
-	    $user = $admin->authTorus()->user1(id => $uid);
-	}; if ($@) {
-	    if (ref($@) eq "HTML::Mason::Exception" && $@ =~ 404) {
-		# Assume the record has been deleted
-		goto DONE;
-	    } else {
-		die $@;
-	    }
+	%criteria = (id => $uid);
+    } else {
+	# We've not been able to log in; but if we're logged in as the
+	# consortium, then we can just act as the user anyway.
+	#
+	# ### I'm unhappy about hardwired knowledge of domains here.
+	#
+	if (loggedInAtDomain($r, 'wizard')) {
+	    warn "wizard finding user by realm '$identityId'";
+	    %criteria = (identityId => $identityId);
 	}
-	die "Invalid user-ID '$uid'" if !defined $user;
+    }
 
-	# Special case: consortium-level administrator is always super
-	$user->{layers}->{final}->{fields}->{superuser} = 1
-	    if $domain eq "wizard";
+    if (!%criteria) {
+	goto DONE;
+    }
+
+    eval {
+	$user = $admin->authTorus()->user1(%criteria);
+    }; if ($@) {
+	if (ref($@) eq "HTML::Mason::Exception" && $@ =~ 404) {
+	    # Assume the record has been deleted
+	    goto DONE;
+	} else {
+	    die $@;
+	}
+    }
+
+    if (!defined $user) {
+	if ($uid) {
+	    die "Invalid user-ID '$uid'";
+	} else {
+	    die "Invalid user-realm '$identityId'";
+	}
+    }
+
+    # Special case: consortium-level administrator is always super
+    $user->setField(superuser => 1) if $domain eq "wizard";
+
+    if (!$uid) {
+	# This session was created from the consortium level: patch it up.
+	warn "patching up wizard-induced session";
+	$session->update(user_id => $user->field('id'));
     }
 
   DONE:
     return ($admin, $session, $user);
 }
 
+
+sub loggedInAtDomain {
+    my($r, $domain) = @_;
+
+    my @links;
+    my $cookies = Apache2::Cookie->fetch($r);
+
+    foreach my $key (sort keys %$cookies) {
+	if ($key =~ /^session-$domain-/) {
+	    my $cookie = $cookies->{$key};
+	    my $sessid = $cookie->value();
+	    push @links, [ $sessid ];
+	}
+    }
+
+    warn "got ", scalar(@links), " sessions in $domain domain";
+    @links = Masterkey::Admin::Session::filter_links(@links);
+    warn "retained ", scalar(@links), " logged-in sessions in $domain domain";
+    return @links ? 1 : 0;
+}
+
 sub siteConf {
     my $this = shift();
     my($key) = @_;
@@ -243,10 +300,13 @@ sub log {
     my $this = shift();
     my($level, @args) = @_;
 
+    my ($seconds, $microseconds) = gettimeofday;
+    my $date = time2str($seconds) . " .${microseconds} ms";
+
     if ($this->option("loglevel") & $level) {
 	### could check another option for whether to include PID
 	my $label = Masterkey::Admin::LogLevel::label($level);
-	print STDERR $this->option("logprefix"), " ($label): ", @args, "\n";
+	print STDERR $this->option("logprefix"), " $date ($label): ", @args, "\n";
     }
 }
 
@@ -332,9 +392,9 @@ sub make_record {
 #
 sub modify_record {
     my $this = shift();
-    my($identity, $id, undef, $goneHash, %data) = @_;
+    my($identity, $id, $master, $goneHash, %data) = @_;
 
-    $this->torus()->update($identity, $id, undef, $goneHash, %data);
+    $this->torus()->update($master, $identity, $id, undef, $goneHash, %data);
     return undef;
 }
 
