@@ -222,6 +222,7 @@ sub sectionHasVisibleFields {
 	    #warn "found visible field '$field'";
 	    return 1;
 	}
+	#warn "ignoring invisible field '$field'";
 	$i++;
     }
 
@@ -250,7 +251,7 @@ sub _open_http {
 	" URL '$url' ", $res->status_line())
 	if !$res->is_success();
 
-    my $text = $res->content();
+    my $text = $res->decoded_content('default_charset' => 'UTF-8');
 
     # We need to return an object that behaves like an IO::File.
     # It has to support getline() and close()
