@@ -22,7 +22,7 @@ sub new {
     my $label = delete $y->{label};
     my $matchQuery = delete $y->{matchQuery};
 
-    my $this = new Masterkey::Admin::Record($torus, $x, $using_XML_LibXML_Simple);
+    my $this = new Masterkey::Admin::Record($torus, 0, $x, $using_XML_LibXML_Simple);
     $this->{id} = $id;
     $this->{label} = $label;
     $this->{matchQuery} = $matchQuery;
